/*****************************************************************************
 *
 * trcapi.h - Transcoder Resource Controler Application Interface
 *
 * This file defines all TRC API definitions.
 * The file automatically includes the file trcdefs.h to define those
 * structures and literal definitions that are common to both the TRC API
 * (client-side) and the transcoder (server side).
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *                      Copyright (c) 2004-2007
 *                by NMS Communications Corporation.
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of NMS Communications.
 *
 *****************************************************************************/

#ifndef _TRCAPI_H
#define _TRCAPI_H

#include "trcdefs.h"            /* common TRC definitions */

/*---------------------------------------------------------------------------*
 * TRC channel handle
 *---------------------------------------------------------------------------*/
#define TRC_HANDLE  void *


/*---------------------------------------------------------------------------*
 * General TRC API literal definitions
 *---------------------------------------------------------------------------*/

/* API version / revision control */
#define TRC_CTL_VERSION         2
#define TRC_CTL_REVISION        1

/* Channel states (as reported by trcInfoVideoChannel) */
#define TRC_STATE_UNAVAIL       0   /* channel is not available (not licensed) */
#define TRC_STATE_AVAILABLE     1   /* channel is available for use */
#define TRC_STATE_CREATING      2   /* channel create in progress */
#define TRC_STATE_STOPPED       3   /* channel created and currently stopped */
#define TRC_STATE_STARTING      4   /* channel start in progress */
#define TRC_STATE_ACTIVE        5   /* channel actively transcoding (has been successfully started) */
#define TRC_STATE_STOPPING      6   /* channel stop in progress */
#define TRC_STATE_DESTROYING    7   /* channel destroy in progress */

/* VTP states (as reported by trcVtpStatus) */
#define TRCVTP_STATE_CONNECTING     1   /* attempting to establish TCP/IP connection */
#define TRCVTP_STATE_CONFIGURING    2   /* passing configuration information to the VTP */
#define TRCVTP_STATE_DISABLED       3   /* connected to VTP; VTP has been disabled (not allowing new channels) */
#define TRCVTP_STATE_ENABLED        4   /* connected to VTP; VTP is ready to create channels (and may already have channels) */
#define TRCVTP_STATE_CONNECT_LOST   5   /* connection to VTP has been lost */
#define TRCVTP_STATE_RECONNECTING   6   /* lost connection re-established. TRC communicating with VTP to synchronize channel states and configuration */
#define TRCVTP_STATE_SHUTTING_DOWN  7   /* TRC is shutting down this VTP connection (due to a call to trcShutdown) */

/* Values converted to string representation by trcValueName */
#define TRCVALUE_RESULT         1   /* TRC result code (including TRC_SUCCESS and TRCERR_xxx) */
#define TRCVALUE_EVENT          2   /* TRC event code (TRCEVN_xxx) */
#define TRCVALUE_CHSTATE        3   /* TRC channel state */
#define TRCVALUE_VTPSTATE       4   /* VTP state */
#define TRCVALUE_TYPE           5   /* TRC channel type (SIMPLEX or FDX) */
#define TRCVALUE_VIDTYPE        6   /* TRC endpoint video type (MPEG-4 or H.263) */
#define TRCVALUE_FRAMERES       7   /* frame resolution (CIF or QCIF) */
#define TRCVALUE_DIRECTION      8   /* direction indicators */
#define TRCVALUE_AVAIL_1        9   /* available for future use */
#define TRCVALUE_AVAIL_2        10  /* available for future use */
#define TRCVALUE_JITTER         11  /* name of the jitter mode */
#define TRCVALUE_PROFILE        12  /* type of profile (SIMPLE or BASELINE) */
#define TRCVALUE_LEVEL          13  /* profile level */
#define TRCVALUE_PACKETIZE      14  /* packetization mode (3016, 2190, 2429) */
#define TRCVALUE_TRACE          15  /* trace type [only 1 trace type bit may be set in value] */
#define TRCVALUE_BIT            16  /* bit setting (TRC_TRUE or TRC_FALSE) */
#define TRCVALUE_OVLEVT         17  /* Async overlay events */
#define TRCVALUE_ENDPOINT       18  /* endpoint identifiers (TRC_END_A or TRC_END_B) */
#define TRCVALUE_PROTOCOL       19  /* communication protocols in use by trancoder (TRC_PROT_xxx) */

/* Configuration commands */
#define TRCCMD_DECODER_CONFIG   1   /* MPEG-4 decoder configuration command */
#define TRCCMD_ENCODER_CONFIG   2   /* MPEG-4 encoder configuration command */

/* Maximum definitions */
#define TRC_MAX_CHANNELS        512 /* maximum number of transcoder channels managed by a single TRC instance */
#define TRC_MAX_VTPS            32  /* maximum number of VTPs that can be managed by any TRC instance */


/*---------------------------------------------------------------------------*
 * Callback function events (asynchronous notifications)
 *---------------------------------------------------------------------------*/

/* asynchronous TRC API function completions [channel-independent] */
#define TRCEVN_SHUTDOWN_DONE        0x004F2001  /* completion of a call to trcShutdown */

/* asynchronous TRC API function completions [channel-specific] */
#define TRCEVN_CREATE_CHANNEL_DONE        0x004F2002  /* completion of a call to trcCreateVideoChannel */
#define TRCEVN_START_CHANNEL_DONE         0x004F2003  /* completion of a call to trcStartVideoChannel */
#define TRCEVN_STOP_CHANNEL_DONE          0x004F2004  /* completion of a call to trcStopVideoChannel */
#define TRCEVN_IFRAME_CHANNEL_DONE        0x004F2005  /* completion of a call to trcIframeVideoChannel */
#define TRCEVN_RESERVED_1                 0x004F2006  /* NO LONGER USED [old TRCEVN_IMAGE_CHANNEL_DONE] */
#define TRCEVN_RESERVED_2                 0x004F2007  /* NO LONGER USED [old TRCEVN_IMAGE_CHANNEL_START] */
#define TRCEVN_RESERVED_3                 0x004F2008  /* NO LONGER USED [old TRCEVN_TEXT_CHANNEL_DONE] */
#define TRCEVN_RESERVED_4                 0x004F2009  /* NO LONGER USED [old TRCEVN_IMAGE_CHANNEL_DONE] */
#define TRCEVN_DESTROY_CHANNEL_DONE       0x004F200A  /* completion of a call to trcDestroyVideoChannel */
#define TRCEVN_CREATE_OVL_DONE            0x004F200B  /* completion of a call to trcCreateOverlay */
#define TRCEVN_DESTROY_OVL_DONE           0x004F200C  /* completion of a call to trcDestroyOverlay */
#define TRCEVN_START_OVL_DONE             0x004F200D  /* completion of a call to trcStartOverlay */
#define TRCEVN_STOP_OVL_DONE              0x004F200E  /* completion of a call to trcStopOverlay */
#define TRCEVN_CHANGE_OVL_CFG_DONE        0x004F200F  /* completion of a call to trcChangeOverlayConfig */
#define TRCEVN_CHANGE_OVL_CONTENT_DONE    0x004F2010  /* completion of a call to trcChangeOverlayContent */
#define TRCEVN_CHANGE_OVL_SCROLLING_DONE  0x004F2011  /* completion of a call to trcChangeOverlayScrolling */

/* unsolicited asynchronous notifications [channel-independent] */
#define TRCEVN_RESOURCE_CHANGE      0x004F2041  /* number of available transcoder resources (port licenses) has changed */

/* unsolicited asynchronous notifications [channel-specific] */
#define TRCEVN_CHANNEL_FAILED       0x004F2081  /* active channel has failed. The application must destroy the channel using trcDestroyVideoChannel */
#define TRCEVN_CHANNEL_LOST         0x004F2082  /* channel connection has been lost. The application can wait for the channel to be automatically recovered by the TRC */
#define TRCEVN_CHANNEL_RECOVERED    0x004F2083  /* channel previously reported as lost has been recovered */
#define TRCEVN_CHANNEL_OVL_EVENT    0x004F2084  /* Overlay related channel event */
#define TRCEVN_CHANNEL_RTCP_BYE     0x004F2085  /* channel has received an RTCP BYE message */
#define TRCEVN_CHANNEL_RTCP_TIMEOUT 0x004F2086  /* channel RTCP layer has detected a timeout related to RTP/RTCP reception */

/*---------------------------------------------------------------------------*
 * TRC API information structures
 *---------------------------------------------------------------------------*/

/* per-channel status summary information */
typedef struct
{
    TRC_HANDLE      trcHandle;  /* TRC_defined handle associated with the channel */
    void           *userKey;    /* user-provided key associated with the channel */
    U32             vtpId;      /* identifies the VTP that the channel has been assigned to */
    tTrcChStatus    status;     /* top-level channel status information */
    U32             vtpChanId;  /* unique channel ID (assigned by the VTP) */

    U8              reserved[28];           /* reserved for future use */
} tTrcChSummary;

/* status summary of all currently defined channels */
typedef struct
{
    U16             chanDefined;            /* total number of channels that have been defined for this instance of the TRC */

    tTrcChSummary   chan[TRC_MAX_CHANNELS]; /* summary-level status information for each defined channel */

    U8              reserved[32];           /* reserved for future use */
} tTrcChAll;

/* per-channel detailed information */
typedef struct
{
    tTrcChSummary   summary;                /* summary-level status information */
    tTrcRes         res;                    /* assigned transcoder resource addressing information */
    tTrcChConfig    config;                 /* channel configuration information (provided when channel started) */
    S8              name[TRC_CHNAME_LEN];   /* ASCII name of the channel (set via trcNameVideoChannel) */

    U8              reserved[2760];         /* reserved for future use */
} tTrcChInfo;

/* per-VTP status summary information */
typedef struct
{
    U32             vtpId;                  /* identifies the VTP that the channel has been assigned to */
    U16             state;                  /* current state of the connection to the VTP (TRCVTP_STATE_xxx) */
    U16             simplexLocal;           /* total number of simplex channels (in use by calling application) assigned to the given VTP */
    U16             fdxLocal;               /* total number of full duplex channels (in use by calling application) assigned to the given VTP */
    tTrcVtpUsage    usage;                  /* total channel usage information and error information for the given VTP */
#if 0
    S8              ipAddr[TRC_IPADDR_LEN]; /* IP address of VTP */
#endif
    U8              reserved[32];           /* reserved for future use */
} tTrcVtpSummary;

/* status summary of all defined VTPs */
typedef struct
{
    U16             vtpDefined;             /* total number of VTPs that have been defined for this instance of the TRC */

    tTrcVtpSummary  vtp[TRC_MAX_VTPS];      /* summary-level status information for each defined VTP */

    U8              reserved[32];           /* reserved for future use */
} tTrcVtpAll;

/* total transcoder resource usage information (across all defined VTPs) */
typedef struct
{
    U16             licensesAvail;          /* total number of transcoder port licenses available */
    U16             vtpDefined;             /* total number of VTPs defined for this instance of the TRC */
    U16             vtpAvail;               /* total number of VTPs that are currently available to the TRC */
    U16             simplexLocal;           /* number of simplex channels currently in use by the calling application */
    U16             fdxLocal;               /* number of full-duplex channels currently in use by the calling application */
    U16             simplexTotal;           /* number of simplex channels currently in use by all applications sharing the same VTP resources */
    U16             fdxTotal;               /* number of full-duplex channels currently in use by all applications sharing the same VTP resources */
    U16             ovlLicense;             /* number of vtps with an overlay license */
    U16             ovlExclusive;           /* number of vtps that only accept channel that request the overlay option */
    U8              reserved[28];           /* reserved for future use */
} tTrcUsage;


/*---------------------------------------------------------------------------*
 * Data array entries provided in asynchronous event messages (callbacks)
 *
 * These definitions list the valid indices into the data array portion
 * of the tTrcMessage structure. The set of valid indices is dependent on
 * the type of event represented by the message.
 *
 * NOTE: Event types that are not listed here do not populate the data array.
 *---------------------------------------------------------------------------*/
#define TRCDATA_COUNT                       16      /* count of TRC message data entries */

/* TRCEVN_IFRAME_CHANNEL_DONE [trcIframeVideoChannel response] */
#define TRCDATA_IFRAME_CHANNEL_DIRECTION    0       /* direction from trcIframeVideoChannel request */

/* TRCEVN_CREATE_OVL_DONE           [trcCreateOverlay response] */
/* TRCEVN_DESTROY_OVL_DONE          [trcDestroyOverlay response] */
/* TRCEVN_START_OVL_DONE            [trcStartOverlay response] */
/* TRCEVN_STOP_OVL_DONE             [trcStopOverlay response] */
/* TRCEVN_CHANGE_OVL_CONFIG_DONE    [trcChangeOvlConfig response] */
/* TRCEVN_CHANGE_OVL_CONTENT_DONE   [trcChangeOvlContent response] */
/* TRCEVN_CHANGE_OVL_SCROLLING_DONE [trcChnageOvlScrolling response] */
#define TRCDATA_OVERLAY_OVLHANDLE           0       /* Handle of target overlay */
#define TRCDATA_OVERLAY_USERKEY             1       /* Application's user key.  */

/* TRCEVN_RESOURCE_CHANGE [unsolicited notification] */
#define TRCDATA_RESOURCE_CHANGE_AVAILABLE   0       /* number of port licenses currently available */
#define TRCDATA_RESOURCE_CHANGE_PREVIOUS    1       /* previous number of licenses that had been available */
#define TRCDATA_RESOURCE_OVL_LICENSE        2       /* Number of vtps that have an overlay license */
#define TRCDATA_RESOURCE_OVL_EXCLUSIVE      3       /* Number of vtps that ONLY allow channels with overlays */

/* TRCEVN_CHANNEL_RTCP_BYE [unsolicited notification] */
#define TRCDATA_CHANNEL_RTCP_ENDPOINT       0       /* endpoint that BYE was received over (TRC_END_A or TRC_END_B) */
#define TRCDATA_CHANNEL_RTCP_DIRECTION      1       /* direction over which BYE was received (TRC_DIR_xxx) */
#define TRCDATA_CHANNEL_RTCP_REASON         2       /* optional reason that was provided with BYE (ASCII string) */

/* TRCEVN_CHANNEL_RTCP_TIMEOUT [unsolicited notification] */
/*      TRCDATA_CHANNEL_RTCP_ENDPOINT       0       @* endpoint that timeout condition has occurred for (TRC_END_A or TRC_END_B) */
/*      TRCDATA_CHANNEL_RTCP_DIRECTION      1       @* direction associated with endpoint connection that has timed out (TRC_DIR_xxx) */
#define TRCDATA_CHANNEL_RTCP_PROTOCOL       2       /* protocol that timeout was detected for (TRC_PROT_RTP or TRC_PROT_RTCP) */

/*---------------------------------------------------------------------------*
 * TRC Asynchronous event message (provided to callback function)
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32             event;                  /* event code (TRCEVN_xxx) */
    U32             result;                 /* result of request or reason for unsolicited indication (TRC_SUCCESS or TRCERR_xxx) */
    TRC_HANDLE      trcChHandle;            /* TRC channel handle associated with the event (only applicable to channel events) */
    void           *userKey;                /* user-supplied key associated with the channel (only applicable to channel events) */
    U32             data[TRCDATA_COUNT];    /* event-specific information (indexes of TRCDATA_xxx) */

    U8              reserved[32];           /* reserved for future use */
} tTrcMessage;


typedef struct
{
    void           *reserved;           /* reserved for future use */
} tTrcUpdateInfo;

#if defined(__cplusplus)
extern "C"
{
#endif // __cplusplus

/*---------------------------------------------------------------------------*
 * Asynchronous event notification function prototype
 *
 * This function is provided by the calling application as a method of
 * servicing all asynchronous events issued by the TRC API.
 *
 * INPUTS: pMsg - TRC API asynchronous notification message
 *         size - size of the notification message (in bytes)
 *
 * OUTPUT: reserved - always return 0 [RESERVED FOR FUTURE USE]
 *
 * NOTE: This function executes as part of the TRC API control thread.
 *       The notification function must use a thread-safe method for
 *       transferring any information to the controlling application's
 *       execution thread.
 *---------------------------------------------------------------------------*/
typedef U32 (* tTrcSendMsg2AppFunc) (tTrcMessage *pMsg, S32 size);

/*---------------------------------------------------------------------------*
 * Application defined logging function prototype
 *
 * This function is provided by the calling application as a method of
 * servicing all logging initiated by the TRC.
 *
 * INPUTS: logline - text to be logged
 *               filehandleindex - RESERVED FOR FUTURE USE
 *
 * OUTPUT: reserved - always returns 0 [RESERVED FOR FUTURE USE]
 *---------------------------------------------------------------------------*/
typedef U32 (* tTrcLogAppFunc) (S8 *logline, U32 filehandleindex);

/*---------------------------------------------------------------------------*
 * Setup function prototypes
 *---------------------------------------------------------------------------*/

/* Initializes the TRC module, initiating connections with all VTPs */
U32 trcInitialize( U16 version, U16 revision, S8 *appName,
                   S8 *configFile, S8 *logFileName, tTrcSendMsg2AppFunc pFunc );

/* Triggers the TRC to begin disconnecting from all available transcoders */
U32 trcShutdown( void );

/* Specifies a user-defined callback function for logging */
U32 trcSetLogFunction(tTrcLogAppFunc pLogFunc);

/*---------------------------------------------------------------------------*
 * Channel Creation function prototypes
 *---------------------------------------------------------------------------*/

/* Creates a simplex or full-duplex transcoder channel */
U32 trcCreateVideoChannel( TRC_HANDLE *trcChHandle, U16 type, void *userKey );

/* Name a previously created transcoder channel */
U32 trcNameVideoChannel( TRC_HANDLE trcChHandle, S8 *chName );

/* Releases a previously created transcoder channel */
U32 trcDestroyVideoChannel( TRC_HANDLE trcChHandle );


/*---------------------------------------------------------------------------*
 * Transcoding function prototypes
 *---------------------------------------------------------------------------*/

/* Starts a transcoder channel, providing description of video transcoding adaptations performed */
U32 trcStartVideoChannel( TRC_HANDLE trcChHandle, tTrcChConfig *chConfig );

/* Causes the transcoder to insert a full image frame information record */
U32 trcIframeVideoChannel( TRC_HANDLE trcChHandle, U16 direction );

/* Stops all transcoding for the specified video channel */
U32 trcStopVideoChannel( TRC_HANDLE trcChHandle );


/*---------------------------------------------------------------------------*
 * Monitoring function prototypes
 *---------------------------------------------------------------------------*/

/* Provides detailed information regarding a specific transcoder channel */
U32 trcInfoVideoChannel( TRC_HANDLE trcChHandle, tTrcChInfo *chInfo );

/* Provides overall channel usage information */
U32 trcUsage( tTrcUsage *usage );

/* Provides overall status of all VTP connections */
U32 trcVTPStatus( tTrcVtpAll *vtpStatus );

/* Provides summary view of all channels currently in use by the calling application */
U32 trcChannelStatus( tTrcChAll *chanStatus );

/* Resets (reboots) the specified transcoder */
U32 trcResetVTP( U32 vtpId );

/* Returns as ASCII string representation of the given numeric value */
S8 *trcValueName( U32 valueType, U32 value );

/* Defines the logging level for the TRC */
void trcSetTrace( U32 logToConsoleMask, U32 logToFileMask );


/*---------------------------------------------------------------------------*
 * Overlay related function prototypes
 *---------------------------------------------------------------------------*/

/* Create a new overlay */
U32 trcCreateOverlay (  TRC_HANDLE trcChHandle,
                        U16 direction,
                        tTrcOvlConfig *ovlConfig,
                        tTrcOvlContent *ovlContent,
                        TRC_OVL_USERKEY ovlUserKey);

/* Destroy existing overlay */
U32 trcDestroyOverlay ( TRC_HANDLE trcChHandle, TRC_OVL_HANDLE ovlHandle);

/* Start an inactive overlay */
U32 trcStartOverlay ( TRC_HANDLE trcChHandle, TRC_OVL_HANDLE ovlHandle);

/* Stop an active overlay */
U32 trcStopOverlay ( TRC_HANDLE trcChHandle, TRC_OVL_HANDLE ovlHandle);

/* Change overlay configuration */
U32 trcChangeOverlayConfig ( TRC_HANDLE trcChHandle,
                             TRC_OVL_HANDLE ovlHandle,
                             tTrcOvlConfig *ovlConfig);

/* Update overlay content */
U32 trcChangeOverlayContent ( TRC_HANDLE trcChHandle,
                              TRC_OVL_HANDLE ovlHandle,
                              U32  options,
                              tTrcOvlContent *ovlContent);

/* Update overlay scrolling configuration */
U32 trcChangeOverlayScrolling ( TRC_HANDLE trcChHandle,
                                TRC_OVL_HANDLE ovlHandle,
                                tTrcOvlScroll *ovlScroll);

/* Retrieve overlay information */
U32 trcGetOverlayInfo ( TRC_HANDLE trcChHandle,
                        TRC_OVL_HANDLE ovlHandle);

/* Update TRC module - adding or removing connections to VTPs*/
U32 trcUpdateVTPConfig(tTrcUpdateInfo *trcUpdateInfo);

#if defined(__cplusplus)
}
#endif // __cplusplus

#endif /* _TRCAPI_H */
