/****************************************************************************
*                              NOTICE
*  This source code contains proprietary and confidential information,
*  and is the property of the Dialogic Corporation.
*
*  This program has been distributed under license agreement and any
*  unauthorized use or distribution is prohibited.
*
*  Copyright 1999 by Dialogic Corporation.  All rights reserved.
 ***************************************************************************/
/****************************************************************************
 * FILE:         trcdefs.h
 * AUTHOR:       
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    04/01/2009
 * LAST-MOD:
 * PURPOSE:      Transcoder Resource Controller - Common Defintions
 *
 * This file defines all TRC API definitions that are common to both the
 * TRC API (client-side) and the transcoder (server side).
 *
 * trcdefs.h is automatically included by trcapi.h
 *
 * HISTORY:
 * Date        Who  CR:Description
 * 07/08/2010  JD   IPY91524:Support for H264 at 2Mbps
 * 11/05/2009  JAF  CR35980: Updating defaults for codec debug/validation
 * 11/02/2009  JAF  CR35817: Adding new defines for IPP debug
 * 10/21/2009  JAF  CR34572: Adding new debug/validation support
 ******************************************************************/

#ifndef _TRCDEFS_H
#define _TRCDEFS_H

#include "trctypes.h"               /* define TRC API basic type definitions */
#include "vserr.h"

/*---------------------------------------------------------------------------*
 * Transcoder completion codes
 *
 * TRC_SUCCESS = requested operation completed successfully
 *
 * TRCWARN_xxx = operation completed successfully, but potential problem detected
 *
 * TRCERR_xxx  = requested operation failed due to reason provided
 *---------------------------------------------------------------------------*/
#define TRC_SUCCESS                     0           /* SUCCESS */

#define TRCWARN_FUTURE_REVISION         0x20000001  /* WARNING: revision of the TRC is older than the revision that the application was built against */
#define TRCWARN_MIN                     TRCWARN_FUTURE_REVISION     /* minimum warning code */
#define TRCWARN_MAX                     TRCWARN_FUTURE_REVISION     /* maximum warning code */

#define TRCERR_SOCKET_FAILURE           0x20010001  /* TRC was unable to open a TCP/IP socket to a VTP or encountered an error while reading data from a VTP socket */
#define TRCERR_FILE_NOT_FOUND           0x20010002  /* referenced file could not be found */
#define TRCERR_INVALID_FILE             0x20010003  /* invalid configuration file provided to trcInitialize or image file reported as invalid (error reported by the VTP) */
#define TRCERR_NORESOURCES              0x20010004  /* no transcoder resources are available */
#define TRCERR_INVALID_CHANNEL_HANDLE   0x20010005  /* TRC channel handle provided does not correspond to a created channel */
#define TRCERR_INVALID_CHANNEL_PARAM    0x20010006  /* one or more of the parameters provided to a TRC function are invalid */
#define TRCERR_FILE_ACCESS_ERROR        0x20010007  /* VTP was unable to access the file indicated */
#define TRCERR_INVALID_FUNC             0x20010008  /* invalid pointer to the callback function provided to trcInitialize */
#define TRCERR_LIB_NOT_INITIALIZED      0x20010009  /* TRC library is not initialized. Call trcInitialize first */
#define TRCERR_INVALID_CHANNEL_STATE    0x2001000A  /* channel is not in a valid state for the requested operation */
#define TRCERR_INVALID_DIRECTION        0x2001000B  /* direction provided to a TRC function is invalid */
#define TRCERR_RESET_FAILED             0x2001000C  /* trcResetVTP was unable to pass the reset request to the VTP */
#define TRCERR_INVALID_IMAGE_NUMBER     0x2001000D  /* image number provided is not a valid image number */
#define TRCERR_INVALID_INPUT_PARAM      0x2001000E  /* parameter provided to trcStartVideoChannel related to transcoder input control is invalid */
#define TRCERR_INVALID_LOCATION         0x2001000F  /* location provide is invalid */
#define TRCERR_INVALID_OUTPUT_PARAM     0x20010010  /* parameter provided to trcStartVideoChannel related to transcoder output control is invalid */
#define TRCERR_INVALID_TEXT_MSG         0x20010011  /* text message is not a valid text string */
#define TRCERR_INVALID_TYPE             0x20010012  /* invalid channel type was provided to trcCreateVideoChannel */
#define TRCERR_INVALID_VERSION          0x20010013  /* version mismatch was detected during a call to trcInitialize */
#define TRCERR_INVALID_VTP_ID           0x20010014  /* VTP identifier provided to the TRC function is not a valid VTP ID */
#define TRCERR_ALREADY_INITIALIZED      0x20010015  /* TRC library has already been initialized */
#define TRCERR_INVALID_APPNAME          0x20010016  /* invalid application name provided */
#define TRCERR_THREAD_USAGE             0x20010017  /* improper thread usage by application - API calls cannot be made from upcall notification */
#define TRCERR_FUNCTION_NOT_SUPPORTED   0x20010018  /* requested function is not currently supported */
#define TRCERR_OUT_OF_MEMORY            0x20010019  /* not enough system memory to complete request */
#define TRCERR_FAILURE                  0x2001001A  /* Generic TRC error */
#define TRCERR_INVALID_VIDEO_TYPE       0x2001001B  /* invalid video type detected (should be H.263, H.264 or MPEG4) */
#define TRCERR_INVALID_PROFILE          0x2001001C  /* invalid profile type detected */
#define TRCERR_INVALID_LEVEL            0x2001001D  /* invalid profile level detected */
#define TRCERR_INVALID_DATA_RATE        0x2001001E  /* invalid data rate detected */
#define TRCERR_INVALID_FRAME_RATE       0x2001001F  /* invalid frame rate detected */
#define TRCERR_INVALID_FRAME_RES        0x20010020  /* invalid frame resolution detected */
#define TRCERR_INVALID_PACKETIZE        0x20010021  /* invalid packetize mode */
#define TRCERR_INVALID_JITTER_MODE      0x20010022  /* invalid input (to the transcoder) jitter mode */
#define TRCERR_INVALID_OUT_IPADDR       0x20010023  /* invalid output (from the transcoder) IP address */
#define TRCERR_INVALID_CONFIG_FILE      0x20010024  /* invalid configuration file */
#define TRCERR_RING_FULL                0x20010025  /* ring buffer full indication */
#define TRCERR_PROCESS_LOST             0x20010026  /* a transcoder process has been lost [unexpected termination] */
#define TRCERR_UNKNOWN_MSG              0x20010027  /* an unknown message type was encountered */
#define TRCERR_UNEXPECTED_MSG           0x20010028  /* a message of known type was encountered when given type not expected */
#define TRCERR_OPERATION_ABORTED        0x20010029  /* given operation has been aborted */
#define TRCERR_PROCESS_ERROR            0x2001002A  /* error detected with the given process */
#define TRCERR_INVALID_OUT_OPTION       0x2001002B  /* invalid output endpoint option */
#define TRCERR_INVALID_OVL_HANDLE       0x2001002C  /* invalid overlay handle. Either not found or TRC_OVL_ALL_OVERLAYS is used on imcompatible service */
#define TRCERR_INVALID_OVL_DATA         0x2001002D  /* invalid overlay content data. */
#define TRCERR_MNG_ABORT                0x2001002E  /* management interface has caused abort */

/* ----- Video Transcoder Error Code generating subsystems ----- */
#define VX_TRC_AGENT_BASE   0x01    /* Video Transcoder TRC Agent error code base */
#define VX_TRC_PROCESS_BASE 0x02    /* Video Transcoder TRC Process error code base */
#define VX_SNMP_BASE        0x03    /* Video Transcoder SNMP error code base */
#define VX_MNG_BASE         0x04    /* Video Transcoder MANAGEMENT error code base */
#define VX_PROC_MON_BASE    0x05    /* Video Transcoder PROCESS MONITOR error code base */

#define VX_TST_POUND_BASE   0x81    /* Video Transcoder test application (vxpound) error code base */
#define VX_TST_AVAIL1_BASE  0x82    /* Video Transcoder test category error code base: AVAILABLE FOR FUTURE USE */
#define VX_TST_GEN_BASE     0x83    /* Video Transcoder test data generator (videogen) error code base */
#define VX_TST_TEST_BASE    0x84    /* Video Transcoder operator-controlled test (vstest) error code base */

/* ----- Overlay event codes ----- */
#define OVL_ERRCODE(unique) VS_ERROR_CODE(VS_SYSTEM_VIDEO_XCODER,VX_TRC_PROCESS_BASE,unique)
#define TRC_OVLEVT_TRCR_RENDER_SUCCESS     (OVL_ERRCODE(0x01))
#define TRC_OVLEVT_VTC_SCROLL_END          (OVL_ERRCODE(0x02))

#define TRC_OVLEVT_VTC_STARTOVL_FAILED     (OVL_ERRCODE(0x51))
#define TRC_OVLEVT_VTC_STOPOVL_FAILED      (OVL_ERRCODE(0x52))
#define TRC_OVLEVT_VTC_SUBMITC_FAILED      (OVL_ERRCODE(0x53))
#define TRC_OVLEVT_VTC_SUBMITF_FAILED      (OVL_ERRCODE(0x54))
#define TRC_OVLEVT_VTC_CREATEOVL_FAILED    (OVL_ERRCODE(0x55))
#define TRC_OVLEVT_VTC_DESTROYOVL_FAILED   (OVL_ERRCODE(0x56))
#define TRC_OVLEVT_TRCR_RENDER_FAILED      (OVL_ERRCODE(0x57))

#define TRC_OVLEVT_TRCP_INVALID_OVL_DATA   (OVL_ERRCODE(VS_REASON_INVALID_CONFIG))
#define TRC_OVLEVT_TRCP_INVALID_OVL_HANDLE (OVL_ERRCODE(VS_REASON_INVALID_KEY))
#define TRC_OVLEVT_TRCP_INVALID_STATE      (OVL_ERRCODE(VS_REASON_INVALID_STATE))
#define TRC_OVLEVT_TRCP_INVALID_TYPE       (OVL_ERRCODE(VS_REASON_INVALID_TYPE))
#define TRC_OVLEVT_TRCP_INVALID_SIZE       (OVL_ERRCODE(VS_REASON_INVALID_SIZE))
#define TRC_OVLEVT_TRCP_INVALID_PKT        (OVL_ERRCODE(VS_REASON_INVALID_PKT))
#define TRC_OVLEVT_TRCP_NOT_SUPPORTED      (OVL_ERRCODE(VS_REASON_NOT_SUPPORTED))
#define TRC_OVLEVT_TRCP_NO_RESOURCE        (OVL_ERRCODE(VS_REASON_NO_RESOURCE))
#define TRC_OVLEVT_TRCP_OUT_OF_MEMORY      (OVL_ERRCODE(VS_REASON_OUT_OF_MEMORY))
#define TRC_OVLEVT_TRCP_SOCKET             (OVL_ERRCODE(VS_REASON_SOCKET))
#define TRC_OVLEVT_TRCP_DECODER            (OVL_ERRCODE(VS_REASON_GET))
#define TRC_OVLEVT_TRCP_ENCODER            (OVL_ERRCODE(VS_REASON_SET))
#define TRC_OVLEVT_TRCP_MAKE_HEADER        (OVL_ERRCODE(VS_REASON_INVALID_HDR))
#define TRC_OVLEVT_TRCP_DOES_NOT_EXIST     (OVL_ERRCODE(VS_REASON_DOES_NOT_EXIST))
#define TRC_OVLEVT_TRCP_OVL_CREATE_FAILED  (OVL_ERRCODE(VS_REASON_FAILED_CREATE))

/*---------------------------------------------------------------------------*
 * Common type definitions
 *---------------------------------------------------------------------------*/

/* ASCII string areas */
#define TRC_IPADDR_LEN      64      /* space reserved for an IP address as ASCII string */
#define TRC_ERROR_INFO_LEN  128     /* space reserved for error info as ASCII string */
#define TRC_TEXT_LEN        512     /* space reserved for timed text ASCII string */
#define TRC_APPNAME_LEN     32      /* space reserved for application name */
#define TRC_CHNAME_LEN      64      /* space reserved for channel name */

/* Channel types */
#define TRC_CH_TYPE_MASK    0x000F  /* mask for channel type */
#define TRC_CH_UNUSED       0x0000  /* channel not currently in use (i.e. not defined) */
#define TRC_CH_FDX          0x0001  /* full-duplex channel (endpoint A <===> endpoint B) */
#define TRC_CH_SIMPLEX      0x0002  /* simplex channel (endpoint A ===> endpoint B) */

#define TRC_CH_REQ_MASK     0xFFF0  /* mask for channel feature requirements */
#define TRC_CH_OVERLAY      0x8000  /* channel requires overlays; restrict to VTPs licensed for overlays */
#define TRC_CH_RTCP         0x0800  /* channel requires RTCP; restrict to VTPs supporting RTCP */

/* Direction indicators */
/* NOTE: direction value of 0 is reserved for future use */
#define TRC_DIR_SIMPLEX     1       /* command applies to only connection leg (endpoint A to endpoint B) */
#define TRC_DIR_FDX1        2       /* command applies to connection leg from endpoint A to endpoint B */
#define TRC_DIR_FDX2        3       /* command applies to connection leg from endpoint B to endpoint A */
#define TRC_DIR_COUNT       4       /* count of possible direction indicators [0-3 (0 reserved for future)] */

/* Endpoint identifiers */
#define TRC_END_A           1       /* related to channel endpoint A */
#define TRC_END_B           2       /* related to channel endpoint B */

/* Communication protocols in use by transcoder */
#define TRC_PROT_RTP        1       /* Real-time Transport Protocol */
#define TRC_PROT_RTCP       2       /* Real-time Transport Control Protocol */

/* Configuration assistance */
#define TRC_TRUE            1
#define TRC_FALSE           0

#define TRC_CONFIG_DEFAULT  0       /* use default mode for given configuration field */

#define TRC_MAX_DATA_RATE   2000     /* maximum supported data rate (in kilobits/sec) */
#define TRC_MAX_FRAME_RATE  30      /* maximum supported frame rate (in frames/sec) */

#define TRC_DFLT_DATA_RATE   43     /* default data rate (in kilobits/sec) */
#define TRC_DFLT_FRAME_RATE  7      /* default frame rate (in frames/sec) */

/* RTCP operating modes */
#define TRC_RTCP_DEFAULT    0       /* handle RTCP as specified by VTP-level default config (rtcpMode) */
#define TRC_RTCP_DISABLED   1       /* do not listen for or send RTCP */
#define TRC_RTCP_ENABLED    2       /* listen for receive of RTCP from remote endpoint and send RTCP to remote as appropriate */

/*---------------------------------------------------------------------------*
 * Common Structures
 *---------------------------------------------------------------------------*/

/* ----- transcoder resource addressing information ----- */
typedef struct
{
    S8              ipAddr[TRC_IPADDR_LEN]; /* IP address of VTP providing transcoder resources */
    U16             rxPort[TRC_DIR_COUNT];  /* assigned transcoder RTP receive port(s) */

    U8              reserved[32];   /* reserved for future use */
} tTrcRes;

/* ----- information describing any error encountered ----- */
typedef struct
{
    U32             trcError;       /* last error encountered expressed as a TRCERR_xxx value */
    U32             osError;        /* operating system specific error code that was mapped to TRC error code */
    S8              errorInfo[TRC_ERROR_INFO_LEN];  /* ASCII description providing detailed information about the error */

    U8              reserved[32];   /* reserved for future use */
} tTrcError;

/* ----- overall error description ----- */
typedef struct
{
    U32             numErrors;      /* total number of errors detected */
    tTrcError       firstError;     /* description of the first error encountered */
    tTrcError       lastError;      /* description of the last (most recent) error encountered */

    U8              reserved[32];   /* reserved for future use */
} tTrcErrorDesc;


/* ----- configuration value that may or may not be set by controlling application ----- */
typedef struct
{
    U8              isSet;          /* TRC_FALSE = value not set by application (use default behavior)
                                     * TRC_TRUE  = value has been set           (use value) */
    U8              avail[3];       /* avaliable for future use */
    U32             value;          /* value (when isSet = TRC_TRUE) [for bit values use TRC_TRUE or TRC_FALSE] */
} tTrcCfgValue;

/*---------------------------------------------------------------------------*
 * Endpoint Structures
 *---------------------------------------------------------------------------*/

/* ----- endpoint info specific to transcoder receiving from endpoint ----- */
typedef struct
{
    U16             jitterMode;     /* mode of operation for receive jitter buffer control */
#define TRC_JITTER_NONE     1       /* no jitter buffer in use */
#define TRC_JITTER_STATIC   2       /* constant jitter buffer offset */

    U32             jitterLatency;  /* jitter buffer latency (in milliseconds): amount of time that received packets are delayed to allow for jitter buffer reordering */
    U8              rtcpReceiver;   /* mode for handling RTCP communication with input endpoint (TRC_RTCP_xxx) */
    U8              avail[3];
    U32             rtcpRxTimeout;  /* max time (in msecs) that can pass without receiving RTP or RTCP before considering endpoint timed out (0 = no timeout) [only valid if rtcpReceiver != 0] */
    U8              reserved[24];   /* reserved for future use */
} tTrcEndInput;

/* ----- endpoint info specific to transcoder transmitting to endpoint ----- */
typedef struct
{
    S8              ipAddr[TRC_IPADDR_LEN]; /* IP address of endpoint that transcoder outputs RTP video bitstream to */
    U16             rtpPort;            /* UDP port number of endpoint that transcoder outputs RTP video bitstream to */
    U32             payloadId;          /* payload ID used in outbound RTP packets issued by the transcoder */
    U8              tos;                /* type of Service used in outbound RTP packets issued by the transcoder */

    tTrcCfgValue    duplicateInitialI;  /* BIT: TRC_TRUE = issue duplicate of initial I-frame */
    tTrcCfgValue    dropEarlyFrames;    /* BIT: TRC_TRUE = encoder will drop early frames */
    tTrcCfgValue    dropLowQualFrames;  /* BIT: TRC_TRUE = encoder will drop low quality frames */
    tTrcCfgValue    timeResolution;     /* (MPEG4 only) time resolution */
    tTrcCfgValue    partitioned;        /* (MPEG4 only) BIT: TRC_TRUE = enable partitioning, TRC_FALSE = disable partitioning */
    tTrcCfgValue    framesPerI;         /* when the encoder generates an I-frame in relation to P-frames
                                         *   0 (default) - Encoder generates only one I-frame.
                                         *                 The other frames are P-frames
                                         *   else        - Frame interval for generating I-frames.
                                         *                 For example, a value of 10 means that the encoder generates
                                         *                 one I-frame and nine P-frames for every ten frames */
    tTrcCfgValue    numMbRefresh;       /* encoder number: macroblock refresh for intra-coding of P-frames (only valid when framesPerI = 0) */
    tTrcCfgValue    packetSize;         /* (MPEG4 only) encoder packet size */
    tTrcCfgValue    timePeriod;         /* whether the encoder uses a fixed time increment for transmitting frames
                                         *   0 (default) - Encoder transmits frames using the time interval
                                         *                 associated with the corresponding decoded input frames
                                         *   else        - count of ticks [as specified by timeResolution]
                                         *                 Encoder transmits frames using the specified time interval */
    tTrcCfgValue    acPrediction;       /* (MPEG4 only) BIT: TRC_TRUE = enable AC prediction */
    tTrcCfgValue    useType2Mb;         /* (MPEG4 only) BIT: TRC_TRUE = enable use of type 2 macroblocks */

    U8              rtcpTransmitter;    /* mode for handling RTCP communication with output endpoint (TRC_RTCP_xxx) */
    U8              avail[3];
    U32             rtcpTxTimeout;      /* max time (in msecs) that can pass without receiving RTCP before considering endpoint timed out (0 = no timeout) [only valid if rtcpTransmitter != 0] */
	tTrcCfgValue    monaSupport;        /* the encoder output is to be optimized for MONA 3G endpoint
										 * monaSupport.isSet = TRC_TRUE implies the encoder output is to be optimized for MONA 3G endpoint
										 * monaSupport.isSet = TRC_FALSE implies the encoder output is not optimized for MONA 3G endpoint	*/
    U8              reserved[24];       /* reserved for future use */
} tTrcEndOutput;

/* ----- complete endpoint configuration ----- */
typedef struct
{
    U16             vidType;        /* identifies the type of video format used by the endpoint */
#define TRC_VIDTYPE_MPEG4       1   /* MPEG4 encoded bit stream */
#define TRC_VIDTYPE_H263        2   /* H.263 encoded bit stream */
#define TRC_VIDTYPE_PIXEL       3   /* pixel encoded bit stream */
#define TRC_VIDTYPE_H264        4   /* H.264 encoded bit stream */

    U16             profile;        /* type of profile in use by endpoint */
#define TRC_PROFILE_SIMPLE      1   /* MPEG-4: simple profile in use */
#define TRC_PROFILE_BASELINE    2   /* H.263: baseline profile in use */

    U16             level;          /* profile level in use by endpoint */
#define TRC_MPEG4_LEVEL_0       1   /* MPEG-4 profile level 0 */
#define TRC_MPEG4_LEVEL_1       2   /* MPEG-4 profile level 1 */
#define TRC_MPEG4_LEVEL_2       3   /* MPEG-4 profile level 2 */
#define TRC_MPEG4_LEVEL_3       4   /* MPEG-4 profile level 3 */
#define TRC_H263_LEVEL_10       5   /* H.263 profile level 10 */
#define TRC_H263_LEVEL_20       6   /* H.263 profile level 20 */
#define TRC_H263_LEVEL_30       7   /* H.263 profile level 30 */
#define TRC_H264_LEVEL_1_0      8   /* H.264 profile level 1 */
#define TRC_H264_LEVEL_1_1      9   /* for H.264 profile level 1.1 */
#define TRC_H264_LEVEL_1b       10  /* for H.264 profile level 1b */
#define TRC_H264_LEVEL_1_2      11  /* for H.264 profile level 1.2 */
#define TRC_H264_LEVEL_1_3      12  /* for H.264 profile level 1.3 */
#define TRC_H264_LEVEL_2_0      13  /* for H.264 profile level 2.0 */
#define TRC_H264_LEVEL_2_1      14  /* for H.264 profile level 2.1 */
#define TRC_H264_LEVEL_2_2      15  /* for H.264 profile level 2.2 */
#define TRC_H264_LEVEL_3_0      16  /* for H.264 profile level 3 */
#define TRC_H263_LEVEL_45       17   /* H.263 profile level 45 */


    U16             dataRate;       /* video bit stream data rate in use by the endpoint expressed in kilobits/second */
    U16             frameRate;      /* video bit stream frame rate in use by the endpoint expressed in frames/second */
    U8              frameRes;       /* video frame resolution */
#define TRC_FRAME_RES_QCIF      1   /* Quarter Common Interchange Format (176 x 144) */
#define TRC_FRAME_RES_CIF       2   /* Common Interchange Format (352 x 288) */
#define TRC_FRAME_RES_SUBQCIF   3   /* Sub-Quarter Common Interchange Format (128 x 96) */

    U8              packetizeMode;  /* packetization mode */
#define TRC_PACKETIZE_2190      1   /* endpoint uses packetization mode defined in RFC 2190 */
#define TRC_PACKETIZE_2429      2   /* endpoint uses packetization mode defined in RFC 2429 */
#define TRC_PACKETIZE_3016      3   /* endpoint uses packetization mode defined in RFC 3016 */

#define TRC_PACKETIZE_3984_SINGLE_NAL_UNIT 	4  /* endpoint uses Single NAL Unit mode for packetizing H.264 as defined in RFC 3984*/
#define TRC_PACKETIZE_3984_NON_INTERLEAVED	5  /* endpoint uses Non-Interleaved mode for packetizing H.264 as defined in RFC 3984*/

    tTrcEndInput    chanIn;         /* characteristics of endpoint when transcoder channel is receiving input from the endpoint */
    tTrcEndOutput   chanOut;        /* characteristics of endpoint when transcoder channel is transmitting output to the endpoint */

    U8              reserved[32];   /* reserved for future use */
} tTrcEndpoint;

/*---------------------------------------------------------------------------*
 * Top-Level Channel Configuration Structures
 *---------------------------------------------------------------------------*/

/* ----- optional configuration information ----- */
typedef struct
{
    void           *optData;        /* pointer to options data record (ex: DCI for MPEG-4) */
    U32             optSize;        /* size of options data (in bytes) */

    U8              reserved[32];   /* reserved for future use */
} tTrcChOptions;

/* ----- channel configuration information (provided when channel started) ----- */
typedef struct
{
    tTrcEndpoint    endpointA;      /* description of endpoint A configuration */
    tTrcEndpoint    endpointB;      /* description of endpoint B configuration */

    tTrcChOptions   decoder[TRC_DIR_COUNT]; /* optional decoder config for each direction */
    tTrcChOptions   encoder[TRC_DIR_COUNT]; /* optional encoder config for each direction */

    U8              reserved[32];   /* reserved for future use */
} tTrcChConfig;

/*---------------------------------------------------------------------------*
 * Status and Information Structures
 *---------------------------------------------------------------------------*/

/* ----- top-level channel status information ----- */
typedef struct
{
    U16             state;          /* current overall state of the channel (TRC_STATE_xxx) */
    U16             type;           /* type of channel (TRC_CH_xxx) */
    tTrcErrorDesc   errorDesc;      /* description of any channel-level errors encountered */

    U8              reserved[32];   /* reserved for future use */
} tTrcChStatus;

/* ----- per-VTP usage information (provided by the VTP-based control agent) ----- */
typedef struct
{
    U16             licensesAvail;  /* total transcoder port licenses available */
    U16             simplexInUse;   /* total simplex channels currently in use by all applications sharing the same VTP */
    U16             fdxInUse;       /* total full duplex channels currently in use by all applications sharing the same VTP */
    tTrcErrorDesc   errorDesc;      /* description of any VTP-level errors encountered */

    U8              reserved[32];   /* reserved for future use */
} tTrcVtpUsage;

/*---------------------------------------------------------------------------*
 * Overlay related definitions
 *---------------------------------------------------------------------------*/

#define TRC_OVL_HANDLE  U32
#define TRC_OVL_USERKEY U32

#define TRC_OVL_STRUCTID_CONFIG         1   /* Overlay config structure */
#define TRC_OVL_STRUCTID_CONTENT        2   /* Overlay content structure */
#define TRC_OVL_STRUCTID_INFO           3   /* Overlay information structure */
#define TRC_OVL_STRUCTID_LIST           4   /* Overlay list structure */
#define TRC_OVL_STRUCTID_CONTENTINFO    5   /* Overlay content information structure */

/* -----  Overlay types -----*/
#define TRC_OVL_TYPE_MULTILINE_TEXT      0   /* Defines a multi-line text overlay*/
#define TRC_OVL_TYPE_SINGLELINE_TEXT     1   /* Defines a single-line text overlay*/
#define TRC_OVL_TYPE_IMAGE               2   /* Defines a Graphic overlay*/

/* -----  Overlay initial states -----*/
#define TRC_OVL_INITSTATE_STARTED       0   /* Overlay is displayed on creation */
#define TRC_OVL_INITSTATE_STOPPED       1   /* Overlay not displayed on creation */

/* -----  Overlay coordinate units -----*/
#define TRC_OVL_COORDINATE_UNIT_UNUSED       0   /* Coordinate is not used.        */
#define TRC_OVL_COORDINATE_UNIT_PIXEL        1   /* Coordinate is given in pixels. */
#define TRC_OVL_COORDINATE_UNIT_PERCENT      2   /* Coordinate represents a percentage of the video frame's resolution. */
#define TRC_OVL_COORDINATE_UNIT_LINECOL      3   /* Coordinate represents a number of lines or columns. */
#define TRC_OVL_COORDINATE_UNIT_MS_PER_PIXEL 4   /* Coordinate represents a number of milliseconds per pixel. */

/* ----- Overlay flexible coordinate structure ----- */
typedef struct
{
    U16                 xUnit;   /* Horizontal coordinate unit (See: TRC_OVL_COORDINATE_UNIT_...)*/
    U16                 yUnit;   /* Vertical coordinate unit (See: TRC_OVL_COORDINATE_UNIT_...)*/
    S32                 x;       /* Horizontal coordinate */
    S32                 y;       /* Vertical coordinate */
} tTrcOvlCoordinates;

/* ----- Font styles ----- */
#define TRC_OVL_FONTSTYLE_NORMAL     0x0000
#define TRC_OVL_FONTSTYLE_BOLD       0x0001
#define TRC_OVL_FONTSTYLE_ITALIC     0x0002
#define TRC_OVL_FONTSTYLE_UNDERLINED 0x0004
#define TRC_OVL_FONTSTYLE_OUTLINED   0x0008

#define TRC_OVL_FONTNAME_MAX   256
#define TRC_OVL_MAX_OVERLAYS   32

/* ----- Basic colors used in overlays ----- */
/*                                  RRGGBBTT */
#define TRC_OVL_COLOR_RED         0xFF0000FF
#define TRC_OVL_COLOR_BLUE        0x0000FFFF
#define TRC_OVL_COLOR_GREEN       0x00FF00FF
#define TRC_OVL_COLOR_YELLOW      0xFFFF00FF
#define TRC_OVL_COLOR_MAGENTA     0xFF00FFFF
#define TRC_OVL_COLOR_CYAN        0x00FFFFFF
#define TRC_OVL_COLOR_WHITE       0xFFFFFFFF
#define TRC_OVL_COLOR_BLACK       0x000000FF
#define TRC_OVL_COLOR_TRANSPARENT 0xFFFFFF00
#define TRC_OVL_COLOR_DEFAULT     0x00000000  /* Special meaning: use default color (not actually transparent) */

#define TRC_OVL_GET_RED( RGBA )         ((RGBA & TRC_OVL_COLOR_RED) >> 24)
#define TRC_OVL_GET_GREEN( RGBA )       ((RGBA & TRC_OVL_COLOR_GREEN) >> 16)
#define TRC_OVL_GET_BLUE( RGBA )        ((RGBA & TRC_OVL_COLOR_BLUE) >> 8)
#define TRC_OVL_GET_ALPHA( RGBA )       (RGBA & 0x000000FF)

#define TRC_OVL_SET_RED( RGBA, R )      {RGBA = (((U32)RGBA & 0x00FFFFFF) | (((U32)R & 0xff)<<24));}
#define TRC_OVL_SET_GREEN( RGBA, G )    {RGBA = (((U32)RGBA & 0xFF00FFFF) | (((U32)G & 0xff)<<16));}
#define TRC_OVL_SET_BLUE( RGBA, B )     {RGBA = (((U32)RGBA & 0xFFFF00FF) | (((U32)B & 0xff)<<8));}
#define TRC_OVL_SET_ALPHA( RGBA, A )    {RGBA = (((U32)RGBA & 0xFFFFFF00) |  ((U32)A & 0xff));}

#define TRC_OVL_SET_RGB( RGBA, R, G, B, A ) {RGBA = (((U32)RGBA & 0x00000000) | (((U32)R & 0xff)<<24) | (((U32)G & 0xff)<<16) | (((U32)B & 0xff)<<8) | ((U32)A & 0xff));}

/* ----- Overlay color typedef ----- */
typedef U32 tTrcOvlColor;

/* ----- Font information structure ----- */
typedef struct
{
    S8                  name[TRC_OVL_FONTNAME_MAX]; /* Name of the font */
    U16                 style;                      /* Normal, Bold, Italic, etc.(See: TRC_OVL_FONTSTYLE_...) */
    U16                 size;                       /* Font size */
    tTrcOvlColor        fgColor;                    /* Foreground color*/
    tTrcOvlColor        bgColor;                    /* Background color*/
} tTrcOvlFont;

/* ----- Border information structure ----- */
typedef struct
{
    U16                 width;                    /* Border width in pixels */
    tTrcOvlColor        color;                    /* Border color*/
} tTrcOvlBorder;

/* ----- Scrolling information structure ----- */
typedef struct
{
    U16                 type;            /* Scrolling type (See: TRC_OVL_SCROLL_TYPE...)*/
    tTrcOvlCoordinates  speedDir;        /* Speed and direction of scrolling */
} tTrcOvlScroll;

/* ----- Overlay configuration structure ----- */
typedef struct
{
    U16                 type;        /* Overlay type (See: TRC_OVL_TYPE_...)*/
    U16                 layer;       /* Overlay display order*/
    U16                 initState;   /* Initial state of the overlay (See: TRC_OVL_INITSTATE_...)*/
    tTrcOvlCoordinates  size;        /* Overlay size */
    tTrcOvlCoordinates  position;    /* Overlay position */
    tTrcOvlColor        fgColor;     /* Default overlay foreground color */
    tTrcOvlColor        bgColor;     /* Default overlay background color */
    tTrcOvlBorder       border;      /* Overlay area border */
    tTrcOvlFont         font;        /* Default font information (only used for text overlays)*/
    U8                  reserved[32];/* Reserved area */
} tTrcOvlConfig;

/* ----- Text overlay scroll types ----- */
#define TRC_OVL_SCROLL_TYPE_NONE        0   /* No scrolling */
#define TRC_OVL_SCROLL_TYPE_CONTENT     1   /* Content based scrolling */
#define TRC_OVL_SCROLL_TYPE_CONTINOUS   2   /* Continuous scrolling */
#define TRC_OVL_SCROLL_TYPE_APPLICATION 3   /* Application controlled scrolling */

/* ----- Text overlay alignment options ----- */
#define TRC_OVL_ALIGN_LEFT        0   /* Text is aligned left */
#define TRC_OVL_ALIGN_RIGHT       1   /* Text is aligned right */
#define TRC_OVL_ALIGN_CENTER      2   /* Text is centered */

/* ----- Content update operations: trcUpdateOverlayContent() ----- */
#define TRC_OVL_CONT_UPDT_OPT_ADD      0x00000001   /* 0:ADD 1:REPLACE */
#define TRC_OVL_CONT_UPDT_OPT_PREPEND  0x00000002   /* 0:APPEND 1:PREPEND */
#define TRC_OVL_CONT_UPDT_OPT_RELOAD   0x00000004   /* 0:Reuse content if possible
                                                       1:Always reload content (bypass cached info) */

/* ----- Overlay content types ----- */
#define TRC_OVL_CONT_TYPE_TEXT      0   /* Text content */
#define TRC_OVL_CONT_TYPE_GRAPHIC   1   /* Graphic content */

/* ----- Text content options ----- */
#define TRC_OVL_TEXT_CONT_WRAP_WORD        0x00000001 /* Word wrap */
#define TRC_OVL_TEXT_CONT_ALIGN_LEFT       0x00000004 /* Align text left  */
#define TRC_OVL_TEXT_CONT_ALIGN_RIGHT      0x00000008 /* Align text right */
#define TRC_OVL_TEXT_CONT_ALIGN_CENTER     0x0000000c /* Center text      */

/* ----- Graphic content options ----- */
#define TRC_OVL_GRAPH_CONT_FIT_HORIZONTAL  0x00000001 /* Reduce image to fit horizontally */
#define TRC_OVL_GRAPH_CONT_FIT_VERTICAL    0x00000002 /* Reduce image to fit vertically */
#define TRC_OVL_GRAPH_CONT_FIT             0x00000003 /* Reduce image to fit overlay    */
#define TRC_OVL_GRAPH_CONT_CENTER          0x0000000c /* Center image in overlay area   */

/* ----- Overlay content structure ----- */
typedef struct
{
    U16                 type;            /* Content type: TRC_OVL_CONT_TYPE_... */
    tTrcOvlScroll       scroll;          /* Scrolling information */
    U32                 options;         /* see TRC_OVL_TEXT_CONT_... and TRC_OVL_GRAPH_CONT_...*/
    tTrcOvlCoordinates  size;            /* Size of rendered content */
    S8                  *ovlData;        /* Pointer to text content or image file URL string. */
} tTrcOvlContent;

/*---------------------------------------------------------------------------*
 * tracing types (valid for TRC API or VTP agent)
 *---------------------------------------------------------------------------*/

/* ----- tracing that is normally active by default ----- */
#define TRCTR_ERR   (1<<0)  /* trace all detected errors */
#define TRCTR_ALM   (1<<1)  /* trace all alarm conditions */

/* ----- application and client/server communication tracing ----- */
#define TRCTR_API   (1<<4)  /* trace the interface between the application and the TRC API */
#define TRCTR_MSG   (1<<5)  /* trace communication messages between the TRC API and VTP agent */

/* ----- informational tracing (by control layer) ----- */
#define TRCTR_THR   (1<<8)  /* trace thread execution control */
#define TRCTR_TOP   (1<<9)  /* trace top-level control information */
#define TRCTR_VTP   (1<<10) /* trace VTP-level control information */
#define TRCTR_CHN   (1<<11) /* trace channel-level control information */

/* ----- FSM tracing ----- */
#define TRCTR_STA   (1<<12) /* trace all state changes */
#define TRCTR_EVT   (1<<13) /* trace all events */
#define TRCTR_ACT   (1<<14) /* trace action routines */

/* ----- core related tracing ----- */
#define TRCTR_TMR   (1<<16) /* trace timers */

/*----- other tracing ----- */
#define TRCTR_HDR   (1<<20)  /* Headers tracing */
#define TRCTR_DBG   (1<<21)  /* Debug tracing */

/* ----- tracing bits that cannot be deactivated ----- */
#define TRCTR_CON   (1<<28) /* always write the trace entry to the console [minimal trace entry format] */
#define TRCTR_FIL   (1<<29) /* always write the trace entry to the log file [minimal trace entry format] */
#define TRCTR_ALL   (TRCTR_CON|TRCTR_FIL)   /* always write to both console and log file [minimal trace entry format] */

/* ----- Levels for codec validation ----- */
#define TRC_CODEC_VALIDATION_NONE		0
#define TRC_CODEC_VALIDATION_HEADER		1
#define TRC_CODEC_VALIDATION_FULL		2

#define TRC_CODEC_VALIDATION_LEVEL_DEFAULT		TRC_CODEC_VALIDATION_HEADER

/* ----- Levels for IPP codec debugging ----- */
/* These are used to translate from values specified in vtp.cfg to
   the values used by the IPPs
   
	#     0x00000001 = trace any detected error
	#     0x00000002 = trace warning indications
	#     0x00000004 = trace upper layer (application) interface
	#     0x00000008 = write bitstream
	#     0x00000010 = write rate control information
	#     0x00000020 = write frame level information
	#     0x00000040 = write macroblock level information
	#     0x00000080 = write block level information
*/

#define TRC_IPPLOG_NONE  0
#define TRC_IPPLOG_ERR   (1 << 0)
#define TRC_IPPLOG_WRN   (1 << 1)
#define TRC_IPPLOG_API   (1 << 2)
#define TRC_IPPLOG_BSI   (1 << 3)
#define TRC_IPPLOG_RCI   (1 << 4)
#define TRC_IPPLOG_FLI   (1 << 5)
#define TRC_IPPLOG_MBI   (1 << 6)
#define TRC_IPPLOG_BLI   (1 << 7)

#define TRC_IPPLOG_DEFAULT		TRC_IPPLOG_NONE
#endif  /* _TRCDEFS_H */
