/*****************************************************************************
 *
 * vslogmsg.h - Vision Server Log Message definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *                      Copyright (c) 1988-2007
 *                by NMS Communications Corporation.
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of NMS Communications Corporation.
 *
 *****************************************************************************/

#ifndef _VSLOGMSG_H
#define _VSLOGMSG_H

/* ----- log-related literals ----- */
#define VSLOG_VERSION       1       /* version and revision of VSLOG control interface and message protocol */
#define VSLOG_REVISION      1

#define VSLOG_TRUE          1
#define VSLOG_FALSE         0

#define VSLOG_MAX_DESC      256     /* space reserved for log-related ASCII text desc. line */
#define VSLOG_MAX_STRING    128     /* space reserved for log-related ASCII location strings */
#define VSLOG_MAX_NAME      32      /* space reserved for log-related ASCII name strings */
#define VSLOG_ID_LEN        (4+2)   /* space reserved for ASCII ID (4-char mini-name) */

/* Direction indicators (for message and binary trace) */
#define VSLOG_DIR_NONE          0   /* direction does not apply */
#define VSLOG_DIR_TX            1   /* trace of message|data transmitted from logging entity */
#define VSLOG_DIR_RX            2   /* trace of message|data received by logging entity */
#define VSLOG_DIR_CNT           3   /* count of possible direction values */

/* index values for outCfg[] and mask[] arrays */
#define VSLOG_INDEX_SYSTEM  0   /* index for system log (outDesc[] and outDef[] arrays) */
#define VSLOG_INDEX_DEBUG   1   /* index for debug log (outDesc[], outDef[] or any mask[] array) */
#define VSLOG_INDEX_TRACE   2   /* index for trace log (outDesc[], outDef[] or any mask[] array) */
#define VSLOG_INDEX_CNT     3

/* remote logging (logging to a remote log manager) */
#define VSLOG_LOGTYPE_ID            1111111111      /* special value used as logType for initial record identifying remote app */

/* event severity values */
#define VSLOG_SEV_EMERGENCY 0x100       /* system is unusable */
#define VSLOG_SEV_ALERT     0x200       /* action must be taken immediately */
#define VSLOG_SEV_CRITICAL  0x300       /* critical conditions */
#define VSLOG_SEV_MAJOR     0x380       /* major affect on system operation */
#define VSLOG_SEV_ERROR     0x400       /* error conditions */
#define VSLOG_SEV_MINOR     0x480       /* minor affect on system operation */
#define VSLOG_SEV_WARNING   0x500       /* warning conditions */
#define VSLOG_SEV_NOTICE    0x600       /* normal but significant condition */
#define VSLOG_SEV_INFO      0x700       /* informational message - no affect on system operation */
#define VSLOG_SEV_DEBUG     0x800       /* debug-level messages */
#define VSLOG_SEV_NONE      0xFFFFFFFF  /* special indicator when no severity provided */

/*---------------------------------------------------------------------------*
 * VSLOG_TIME - timestamp record
 *---------------------------------------------------------------------------*/
typedef struct __vslog_time
{
    U32         year;
    U16         month;
    U16         day;
    U16         hour;
    U16         min;
    U16         sec;
    U32         msecs;
} VSLOG_TIME;

/*---------------------------------------------------------------------------*
 * VSLOG_RECORD - header record describing log entry
 *                (written to binary trace files [and to remote log manager])
 *---------------------------------------------------------------------------*/
typedef struct __vslog_record
{
    U32             recStartId;         /* ASCII ID indicating start of a new log record header */
#define VSLOG_START_ID      0x4C523E3E  /* 'LR>>' */
    U8              recVersion;         /* version of VSLOG message protocol */
    U8              recRevision;        /* revision of VSLOG message protocol */
    U8              avail1[2];          /* get back to even 4-byte alignment */

    U32             sequenceNumber;     /* used when forwarding records between endpoints */

    VSLOG_TIME      timestamp;          /* timestamp when record was generated */
    S8              hostName[VSLOG_MAX_STRING]; /* name of host where record was generated */
    S8              systemID[VSLOG_ID_LEN];     /* short name ID'ing system */
    S8              systemName[VSLOG_MAX_NAME]; /* full name of system (process name) */
    U32             systemInst;         /* unique system instance value */
    S8              subsysID[VSLOG_ID_LEN];     /* short name ID'ing subsystem */
    S8              entName[VSLOG_MAX_NAME];    /* name of the logging entity (subsystem-specific name) */
    S8              locName[VSLOG_MAX_NAME];    /* name of location that generated log record */
    U32             logMask;            /* mask that caused entry to be logged (debugMask or traceMask) */
    S8              bitName[VSLOG_MAX_NAME];    /* ASCII name of log bit triggering log entry output */
    U32             direction;          /* direction associated with record (VSLOG_DIR_xxx) */
    U32             logType;            /* type of log record (VSLOG_INDEX_xxx [or VSLOG_LOGTYPE_ID]) */
    U32             payloadLen;         /* byte length of record payload (immed'ly follows VSLOG_RECORD) */

    U32             eventId;            /* value identifying type of event that has occurred */
    U32             eventSeverity;      /* indication of how severe an event is (lower values = more severe) [VSLOG_SEV_xxx] */
    U32             binRecType;         /* type of record (externally controlled for binary trace) */
    S8              textDesc[VSLOG_MAX_DESC];   /* ASCII text description associated with log record */
    U8              reserved[64];       /* reserved for future use */

    U32             recEndId;           /* ASCII ID indicating end of log record header */
#define VSLOG_END_ID        0x3C3C4C52  /* '<<LR' */

    /* vvvvv record immediately followed by log entry payload (for payloadLen) vvvvv */

} VSLOG_RECORD;

#endif /* _VSLOGMSG_H */
