/*****************************************************************************
 *
 * vtmng.h - Video Transcoder Management Interface
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *                      Copyright (c) 1988-2007
 *                by NMS Communications Corporation.
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of NMS Communications Corporation.
 *
 *****************************************************************************/

#ifndef _VTMNG_H
#define _VTMNG_H

/* ----- dependent includes ----- */
#include "vserr.h"                  /* error code base defintions */
#include "vstypes.h"                /* define basic type definitions */
#include "transmanage.h"            /* video transcoder management message definitions */

/* ----- VTMNG definitions ----- */

/*****************************************************************************
 * Redefinition of common trace bits (usually set by default)
 *****************************************************************************/
#define VTMNG_LOG_ERR       (1<<0)                  /* Error detected */
#define VTMNG_LOG_WRN       (1<<1)                  /* Warning condition */

/*****************************************************************************
 * Video Transcoder Management trace bits
 *****************************************************************************/
#define VTMNG_LOG_MIN   16  /* minimum VTMNG API - specific debug/trace type */
#define VTMNG_LOG_MAX   23  /* maximum VTMNG API - specific debug/trace type */

#define VTMNG_LOG_VTP       (1<<(VTMNG_LOG_MIN))    /* trace VTP-level threshold crossed events */
#define VTMNG_LOG_APP       (1<<(VTMNG_LOG_MIN+1))  /* trace application connection events */
#define VTMNG_LOG_CHN       (1<<(VTMNG_LOG_MIN+2))  /* trace channel state change indications */
#define VTMNG_LOG_MON       (1<<(VTMNG_LOG_MIN+3))  /* trace monitored process indications */
#define VTMNG_LOG_E2C       (1<<(VTMNG_LOG_MIN+4))  /* trace event information to console */
#define VTMNG_LOG_CHD       (1<<(VTMNG_LOG_MIN+5))  /* trace detailed channel-level information */
#define VTMNG_LOG_MNG       (1<<(VTMNG_LOG_MIN+6))  /* trace management messages (summaries) */
#define VTMNG_LOG_FUL       (1<<(VTMNG_LOG_MIN+7))  /* trace management messages (full messages) */

/*****************************************************************************
 * special port options
 *****************************************************************************/
#define VTMNG_PORT_SELECT   0xFFFFFFFF  /* select any available port */
#define VTMNG_PORT_DISABLED 0           /* capability disabled; will not register for receives over any port */

/*****************************************************************************
 * values that can be provided to vtMngValueName
 *****************************************************************************/
#define VTMNG_VALUE_RESULT      0x01        /* result presented through management interface */
#define VTMNG_VALUE_LASTERR     0x02        /* last error code encountered (any lastError management statistic) */

/*****************************************************************************
 * Video Transcoder Platform (VTP) addressing information
 *****************************************************************************/
typedef struct __vtmng_vtpaddr
{
    S8      hostName[VTMNG_LONGNAME_SZ];    /* name of VTP ("1.2.3.4" or "vtp1") */
    U32     ipv4Addr;                       /* IPv4 address */
    U32     sendkey;                        /* used as senderID for requests; gives received senderID for responses and notifications */
    U8      reserved[32];                   /* reserved for future use */
} VTMNG_VTPADDR;

/*****************************************************************************
 * upcall (callback) routine prototypes
 *****************************************************************************/

/* ----- responses providing lists of elements ----- */
typedef void  (* vtMngGetAppListRsp)    ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ENT_ID             *app,
                                          U32                       appCount );
typedef void  (* vtMngGetMonListRsp)    ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ENT_ID             *mon,
                                          U32                       monCount );
typedef void  (* vtMngGetChnListRsp)    ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ENT_ID             *chn,
                                          U32                       chnCount );

/* ----- responses providing details of a specific element ----- */
typedef void  (* vtMngGetVtpRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_VTP_CFG            *cfg,
                                          VTMNG_VTP_STATUS         *status,
                                          VTMNG_VTP_STATS          *stats );
typedef void  (* vtMngGetAppRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       appUnique,
                                          VTMNG_APP_CFG            *cfg,
                                          VTMNG_APP_STATUS         *status,
                                          VTMNG_APP_STATS          *stats );
typedef void  (* vtMngGetMonRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       monUnique,
                                          VTMNG_MON_CFG            *cfg,
                                          VTMNG_MON_STATUS         *status,
                                          VTMNG_MON_STATS          *stats );
typedef void  (* vtMngGetChnRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       chnUnique,
                                          VTMNG_CHN_CFG            *cfg,
                                          VTMNG_CHN_STATUS         *status,
                                          VTMNG_CHN_STATS          *stats );
typedef void  (* vtMngGetStTotalRsp)    ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ST_ENTRY           *stats );
typedef void  (* vtMngGetStCurrMinRsp)  ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ST_ENTRY           *stats );
typedef void  (* vtMngGetHistPerMinRsp) ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       minutesAgo,
                                          VTMNG_ST_ENTRY           *stats );
typedef void  (* vtMngGetHistPerHHrRsp) ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32               halfHoursAgo,
                                          VTMNG_ST_ENTRY           *stats );

/* ----- responses to configuration (or overall state) modifications ----- */
typedef void  (* vtMngSetVtpRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_VTP_CFG            *cfg );
typedef void  (* vtMngEventVtpRsp)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       vtpEvent,
                                          VTMNG_VTP_CFG            *cfg );
typedef void  (* vtMngSetAppRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       appUnique,
                                          VTMNG_APP_CFG            *cfg );
typedef void  (* vtMngEventAppRsp)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       appUnique,
                                          U32                       appEvent,
                                          VTMNG_APP_CFG            *cfg );
typedef void  (* vtMngSetMonRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       monUnique,
                                          VTMNG_MON_CFG            *cfg );
typedef void  (* vtMngEventMonRsp)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       monUnique,
                                          U32                       monEvent,
                                          VTMNG_MON_CFG            *cfg );
typedef void  (* vtMngSetChnRsp)        ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       chnUnique,
                                          VTMNG_CHN_CFG            *cfg );
typedef void  (* vtMngEventChnRsp)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       chnUnique,
                                          U32                       chnEvent,
                                          VTMNG_CHN_CFG            *cfg );

/* ----- responses to zero-statistics requests ----- */
typedef void  (* vtMngZeroVtpRsp)       ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_VTP_STATS          *vtpStats );
typedef void  (* vtMngZeroAppRsp)       ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       appUnique,
                                          VTMNG_APP_STATS          *appStats );
typedef void  (* vtMngZeroMonRsp)       ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       monUnique,
                                          VTMNG_MON_STATS          *monStats );
typedef void  (* vtMngZeroChnRsp)       ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          U32                       chnUnique,
                                          VTMNG_CHN_STATS          *chnStats );
typedef void  (* vtMngZeroTotalRsp)     ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg, U32 result,
                                          VTMNG_ST_ENTRY           *stats );

/* ----- traps (unsolicited asynchronous notifications) ----- */
typedef void  (* vtMngVtpLevelTrap)     ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_VTPLVL_NOTIF       *vtpLvl );
typedef void  (* vtMngVtpErrorTrap)     ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          VTMNG_NOTIF_INFO         *info );
typedef void  (* vtMngAppConnTrap)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       appUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_APPCON_NOTIF       *appCon );
typedef void  (* vtMngMonProcTrap)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       monUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_MONPROC_NOTIF      *monProc );
typedef void  (* vtMngChnCreateTrap)    ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       chnUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_CHN_CREATE_NOTIF   *chnCreate );
typedef void  (* vtMngChnStartTrap)     ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       chnUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_CHN_START_NOTIF    *chnStart );
typedef void  (* vtMngChnStopTrap)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       chnUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_CHN_STOP_NOTIF     *chnStop );
typedef void  (* vtMngChnDeadTrap)      ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       chnUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_CHN_DEAD_NOTIF     *chnDead );
typedef void  (* vtMngChnErrorTrap)     ( void *userkey, VTMNG_VTPADDR *vtpAddr, VT_MNG_MSG *msg,
                                          U32                       chnUnique,
                                          VTMNG_NOTIF_INFO         *info,
                                          VTMNG_CHN_ERROR_NOTIF    *chnError );

/* ----- optional keyboard input handling ----- */
typedef U8    (* vtMngCmdNotif)         ( void *userkey, S8     *command );

/*****************************************************************************
 * upcall function set
 *****************************************************************************/
typedef struct __vtmng_upcalls
{
    /* ----- responses providing lists of elements ----- */
    vtMngGetAppListRsp      vtGetAppListRsp;
    vtMngGetMonListRsp      vtGetMonListRsp;
    vtMngGetChnListRsp      vtGetChnListRsp;

    /* ----- responses providing details of a specific element ----- */
    vtMngGetVtpRsp          vtGetVtpRsp;
    vtMngGetAppRsp          vtGetAppRsp;
    vtMngGetMonRsp          vtGetMonRsp;
    vtMngGetChnRsp          vtGetChnRsp;
    vtMngGetStTotalRsp      vtGetStTotalRsp;
    vtMngGetStCurrMinRsp    vtGetStCurrMinRsp;
    vtMngGetHistPerMinRsp   vtGetHistPerMinRsp;
    vtMngGetHistPerHHrRsp   vtGetHistPerHHrRsp;

    /* ----- responses to configuration (or overall state) modifications ----- */
    vtMngSetVtpRsp          vtSetVtpRsp;
    vtMngEventVtpRsp        vtEventVtpRsp;
    vtMngSetAppRsp          vtSetAppRsp;
    vtMngEventAppRsp        vtEventAppRsp;
    vtMngSetMonRsp          vtSetMonRsp;
    vtMngEventMonRsp        vtEventMonRsp;
    vtMngSetChnRsp          vtSetChnRsp;
    vtMngEventChnRsp        vtEventChnRsp;

    /* ----- responses to zero-statistics requests ----- */
    vtMngZeroVtpRsp         vtZeroVtpRsp;
    vtMngZeroAppRsp         vtZeroAppRsp;
    vtMngZeroMonRsp         vtZeroMonRsp;
    vtMngZeroChnRsp         vtZeroChnRsp;
    vtMngZeroTotalRsp       vtZeroTotalRsp;

    /* ----- traps (unsolicited asynchronous notifications) ----- */
    vtMngVtpLevelTrap       vtVtpLevelTrap;
    vtMngVtpErrorTrap       vtVtpErrorTrap;
    vtMngAppConnTrap        vtAppConnTrap;
    vtMngMonProcTrap        vtMonProcTrap;
    vtMngChnCreateTrap      vtChnCreateTrap;
    vtMngChnStartTrap       vtChnStartTrap;
    vtMngChnStopTrap        vtChnStopTrap;
    vtMngChnDeadTrap        vtChnDeadTrap;
    vtMngChnErrorTrap       vtChnErrorTrap;

    /* ----- optional keyboard input handling ----- */
    vtMngCmdNotif           vtCmdNotif;

} VTMNG_UPCALLS;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *
 * PROTOTYPES
 *
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * EXECUTION CONTROL
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/******************************************************************************
 * vtMngInit - initialize and activate management communication interface
 *
 * INPUTS: userkey  - user-controlled key to be provided on all notifications
 *         upcalls  - set of upcall functions (received responses|traps [keyboard])
 *         reqPort  - UDP port number to use for issuing requests (and receiving responses)
 *         trapPort - UDP port number to listen on for receiving traps
 *         dbgmask  - mask of active trace bits
 *                      BIT  0: trace any errors encountered
 *                      BIT  1: trace any warnings encountered
 *                      BIT 16: trace async events to console
 *                      BIT 23: trace management message send/receive to debug log
 *         initdest - OPTIONAL name of initial destination
 *                    (used for internally generated management requests)
 *         eventlog - OPTIONAL filename used to log async events to disk
 *
 * OUTPUT: result of management initialization
 ******************************************************************************/
U32 vtMngInit( void *userkey, VTMNG_UPCALLS *upcalls,
               U32 reqPort, U32 trapPort,
               U32 dbgMask,
               S8 *initdest, S8 *eventlog );

/******************************************************************************
 * vtMngPollLoop - enter polling loop, upcall on receives (management or keyboard)
 *
 * INPUTS: keyboard - flag: whether to register for keyboard input (operator commands)
 *         options  - reserved for future use [must be set to 0]
 *
 * OUTPUT: reason for exiting poll loop
 *           VS_SUCCESS = operator specified "quit"
 *           VTMNG_WRN_TIMEOUT = idle timeout period elapsed without activity
 ******************************************************************************/
U32 vtMngPollLoop( U8 keyboard, U32 options );

/******************************************************************************
 * vtMngShutdown - shutdown all usage of the Video Transcoder Management interface
 *
 * INPUTS: NONE
 *
 * OUTPUT: result of shutdown
 ******************************************************************************/
U32 vtMngShutdown( );

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * MANAGEMENT REQUESTS
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/******************************************************************************
 * vtMngGetAppList - Issue a request for the list of connected applications
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetAppList( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetMonList - Issue a request for the list of monitored processes
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetMonList( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetChnList - Issue a request for the list of defined channels
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetChnList( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetVtp - Issue a request for all VTP-level information
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetVtp( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetApp - Issue a request for details of a particular application
 *
 * INPUTS: vtpAddr   - VTP address
 *         appUnique - application ID value (unique across all apps connected to VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetApp( VTMNG_VTPADDR *vtpAddr, U32 appUnique );

/******************************************************************************
 * vtMngGetMon - Issue a request for details of a particular monitored process
 *
 * INPUTS: vtpAddr   - VTP address
 *         monUnique - monitored process ID value (unique across all monitored processes on VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetMon( VTMNG_VTPADDR *vtpAddr, U32 monUnique );

/******************************************************************************
 * vtMngGetChn - Issue a request for details of a particular channel
 *
 * INPUTS: vtpAddr - VTP address
 *         chnUnique - channel ID value (unique across all chans connected to VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetChn( VTMNG_VTPADDR *vtpAddr, U32 chnUnique );

/******************************************************************************
 * vtMngGetStTotal - Issue a request for the total (overall) statistics
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetStTotal( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetStCurrMin - Issue a request for the current minute's statistics
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetStCurrMin( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngGetHistPerMin - Issue a request for a historical statistics record (per-minute)
 *
 * INPUTS: vtpAddr    - VTP address
 *         minutesAgo - number of minutes into the past history
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetHistPerMin( VTMNG_VTPADDR *vtpAddr, U32 minutesAgo );

/******************************************************************************
 * vtMngGetHistPerHHr - Issue a request for a historical statistics record (per-half-hour)
 *
 * INPUTS: vtpAddr      - VTP address
 *         halfHoursAgo - number of half hour intervals into the past history
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngGetHistPerHHr( VTMNG_VTPADDR *vtpAddr, U32 halfHoursAgo );

/******************************************************************************
 * vtMngSetVtp - Issue a request to modify the VTP-level configuration
 *
 * INPUTS: vtpAddr - VTP address
 *         cfg     - set of configuration changes
 *         mask    - indication of which fields are being altered
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngSetVtp( VTMNG_VTPADDR *vtpAddr, VTMNG_VTP_CFG *cfg, VTMNG_VTP_CFG *mask );

/******************************************************************************
 * vtMngEventVtp - Issue a request to perform a VTP-level event
 *
 * INPUTS: vtpAddr  - VTP address
 *         vtpEvent - VTP-level event to be issued
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngEventVtp( VTMNG_VTPADDR *vtpAddr, U32 vtpEvent );

/******************************************************************************
 * vtMngSetApp - Issue a request to modify a particular application's configuration
 *
 * INPUTS: vtpAddr   - VTP address
 *         appUnique - application ID value (unique across all apps connected to VTP)
 *         cfg       - set of configuration changes
 *         mask      - indication of which fields are being altered
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngSetApp( VTMNG_VTPADDR *vtpAddr, U32 appUnique, VTMNG_APP_CFG *cfg, VTMNG_APP_CFG *mask );

/******************************************************************************
 * vtMngEventApp - Issue a request to perform an application-level event
 *
 * INPUTS: vtpAddr   - VTP address
 *         appUnique - application ID value (unique across all apps connected to VTP)
 *         appEvent  - application-level event to be issued
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngEventApp( VTMNG_VTPADDR *vtpAddr, U32 appUnique, U32 appEvent );

/******************************************************************************
 * vtMngSetMon - Issue a request to modify a particular monitored process configuration
 *
 * INPUTS: vtpAddr   - VTP address
 *         monUnique - monitored process ID value (unique across all monitored processes on VTP)
 *         cfg       - set of configuration changes
 *         mask      - indication of which fields are being altered
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngSetMon( VTMNG_VTPADDR *vtpAddr, U32 monUnique, VTMNG_MON_CFG *cfg, VTMNG_MON_CFG *mask );

/******************************************************************************
 * vtMngEventMon - Issue a request to perform a process monitor event
 *
 * INPUTS: vtpAddr   - VTP address
 *         monUnique - monitored process ID value (unique across all monitired processes on VTP)
 *         monEvent  - monitored process event to be issued
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngEventMon( VTMNG_VTPADDR *vtpAddr, U32 monUnique, U32 monEvent );

/******************************************************************************
 * vtMngSetChn - Issue a request to modify a particular channel's configuration
 *
 * INPUTS: vtpAddr   - VTP address
 *         chnUnique - channel ID value (unique across all chans connected to VTP)
 *         cfg       - set of configuration changes
 *         mask      - indication of which fields are being altered
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngSetChn( VTMNG_VTPADDR *vtpAddr, U32 chnUnique, VTMNG_CHN_CFG *cfg, VTMNG_CHN_CFG *mask );

/******************************************************************************
 * vtMngEventChn - Issue a request to perform a channel-level event
 *
 * INPUTS: vtpAddr   - VTP address
 *         chnUnique - channel ID value (unique across all chans connected to VTP)
 *         chnEvent  - channel-level event to be issued
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngEventChn( VTMNG_VTPADDR *vtpAddr, U32 chnUnique, U32 chnEvent );

/******************************************************************************
 * vtMngZeroVtp - Issue a request to get and then zero the VTP-level statistics
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngZeroVtp( VTMNG_VTPADDR *vtpAddr );

/******************************************************************************
 * vtMngZeroApp - Issue a request to get and then zero an application's statistics
 *
 * INPUTS: vtpAddr   - VTP address
 *         appUnique - application ID value (unique across all apps connected to VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngZeroApp( VTMNG_VTPADDR *vtpAddr, U32 appUnique );

/******************************************************************************
 * vtMngZeroMon - Issue a request to get and then zero a monitored process' statistics
 *
 * INPUTS: vtpAddr   - VTP address
 *         monUnique - monitired process ID value (unique across all monitired processes on VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngZeroMon( VTMNG_VTPADDR *vtpAddr, U32 monUnique );

/******************************************************************************
 * vtMngZeroChn - Issue a request to get and then zero a channel's statistics
 *
 * INPUTS: vtpAddr   - VTP address
 *         chnUnique - channel ID value (unique across all chans connected to VTP)
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngZeroChn( VTMNG_VTPADDR *vtpAddr, U32 chnUnique );

/******************************************************************************
 * vtMngZeroTotal - Issue a request to get and then zero the total statistics
 *
 * INPUTS: vtpAddr - VTP address
 *
 * OUTPUT: result of sending the request
 ******************************************************************************/
U32 vtMngZeroTotal( VTMNG_VTPADDR *vtpAddr );

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 *
 * RAW MESSAGE ACCESS [NOT NEEDED WHEN HIGHER LAYER INTERFACE IN USE]
 *
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/******************************************************************************
 * vtMngMsg2Network - Converts message from host byte order to network byte order
 *
 * INPUTS: msg - message to be converted
 *         len - total byte length of message to be converted
 *
 * OUTPUT: result of conversion
 ******************************************************************************/
U32 vtMngMsg2Network( VT_MNG_MSG *msg, U32 len );

/******************************************************************************
 * vtMngMsg2Host - Converts message from network byte order to host byte order
 *
 * INPUTS: msg - message to be converted
 *         len - total byte length of message to be converted
 *
 * OUTPUT: result of conversion
 ******************************************************************************/
U32 vtMngMsg2Host( VT_MNG_MSG *msg, U32 len );

/******************************************************************************
 * vtMngMsgSize - return total byte size of message
 *
 * INPUTS: msgOp       - message operation (VTMNG_OP_xxx)
 *         msgTypeId   - message type ID (VTMNG_xxx_ID)
 *         msgCategory - message category (VTMNG_CATEG_xxx)
 *         eventId     - event [async indications only]; else use 0
 *         count       - number of elements [lists only]; else use 0
 *
 * OUTPUT: byte length of message
 ******************************************************************************/
U32 vtMngMsgSize( U8 msgOp, U8 msgTypeId, U8 msgCategory,
                  U32 eventId, U32 count );

/******************************************************************************
 * vtMngValueName - return ASCII name of give value code (of specified value type)
 *
 * INPUTS: valueType - VTMNG_VALUE_xxx literal indicating type of value to be named
 *         value     - value to provide ASCII name of
 *
 * OUTPUT: pointer to ASCII string representation of the given value
 ******************************************************************************/
S8 *vtMngValueName( U32 valueType, U32 value );

#endif /* _VTMNG_H */
